#!/usr/bin/python3

# Copyright (C) 2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

# Check that pyknx is present as soon as possible.
from homewatcher import ensurepyknx

from pyknx import logger
import subprocess
import os
import threading
import time

class Timer(threading.Thread):
    def __init__(self, sensor, timeout, name, onTimeoutReached, onIterate = None, onTerminated = None):
        threading.Thread.__init__(self, name=name + ' (id={0})'.format(id(self)))
        self.sensor = sensor
        self.timeout = timeout
        self.reset()
        self.isTerminating = False
        self.isTerminated = False # Mainly for unit testing.
        self.onIterate = onIterate
        self.onTimeoutReached = onTimeoutReached
        self.onTerminated = onTerminated

    def run(self):
        try:
            logger.reportDebug('Starting {0}'.format(self))
            while not self.isTerminating:
                if self.onIterate is not None: self.onIterate(self)

                # Check for termination.
                if self.isTerminating:
                    return

                # Main loop.
                if not self.isPaused:
                    if self.endTime is None: self.extend(starts=True)
                    if time.time() > self.endTime:
                        # Execute delayed job.
                        logger.reportDebug('Timeout reached for {0}.'.format(self))
                        if callable(self.onTimeoutReached): self.onTimeoutReached(self)
                        break

                time.sleep(0.2)
        finally:
            if self.isTerminating:
                # Timer has been stopped from outside.
                logger.reportDebug('{0} is canceled.'.format(self))
            else:
                # Maybe useless but set it for consistency.
                self.isTerminating = True
            if callable(self.onTerminated): self.onTerminated(self)
            logger.reportDebug('{0} is now terminated.'.format(self))
            self.isTerminated = True

    def forceTimeout(self):
        logger.reportDebug('Forcing timeout of {0}'.format(self))
        self.endTime = 0

    def pause(self):
        self.isPaused = True

    def stop(self):
        if not self.isTerminating:
            self.isTerminating = True
            logger.reportDebug('Stopping {0}.'.format(self))

    def reset(self):
        self.endTime = None
        self.isPaused = False

    def __str__(self):
        return '{0} => {1} id={2}'.format(self.sensor, self.name, id(self))

    def extend(self, starts = False):
        """ Prolong duration by the timeout amount of time. """
        if self.isTerminating:
            raise Exception('Timer {0} is terminating, it cannot be extended.'.format(self))
        self.endTime = time.time() + self.timeout
        if starts:
            logger.reportDebug('{0} started for {1} seconds.'.format(self, self.timeout))
        else:
            logger.reportDebug('{0} is extended by {1} seconds.'.format(self, self.timeout))



#!/usr/bin/python3

# Copyright (C) 2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

# Check that pyknx is present as soon as possible.
from homewatcher import ensurepyknx

from pyknx import logger
from homewatcher import configuration, alarm

alarmDaemon = None

def initializeUserScript(context):
    global alarmDaemon
    logger.reportInfo('hw config is {0}'.format(context.hwconfig))
    # motionConfigDir = context.customArgs.get('motionconfigdir')
    # motionOutputDir = context.customArgs.get('motionoutputdir')

    if isinstance(context.hwconfig, str):
        config = configuration.Configuration.parseFile(context.hwconfig)
    elif isinstance(context.hwconfig, configuration.Configuration):
        config = context.hwconfig
    else:
        raise Exception('The hwconfig argument must be either a string or a homewatcher.configuration.Configuration object. "{0}" was passed.'.format(type(context.hwconfig)))

    # Instanciate daemon.
    alarmDaemon = alarm.Daemon(context.communicator, config)

def finalizeUserScript(context):
    global alarmDaemon
    if alarmDaemon:
        alarmDaemon.terminate()

def endUserScript(context):
    global alarmDaemon
    alarmDaemon = None

def onModeObjectChanged(context):
    global alarmDaemon
    logger.reportDebug('Alarm mode changed to ' + str(context.object.value))
    alarmDaemon.onModeValueChanged(context.object.value)

def onWatchedObjectChanged(context):
    global alarmDaemon
    sensor = alarmDaemon.notifyWatchedObjectChanged(context.objectId)

def onSirenStatusChanged(context):
    global alarmDaemon
    alarmDaemon.onSirenStatusChanged(context)

def onAlertPersistenceObjectChanged(context):
    global alarmDaemon
    alarmDaemon.onPersistentAlertChanged(context.object)

def onAlertInhibited(context):
    global alarmDaemon
    alarmDaemon.onAlertInhibited(context.objectId)

def onTemperatureChanged(context):
    global alarmDaemon
    sensor = alarmDaemon.findSensorByTemperatureObjectId(context.objectId)
    temp = float(context.object.value)
    if temp < sensor.lowLimit:
        sensor.triggerObject.value = True
    elif temp > sensor.highLimit:
        sensor.triggerObject.value = True
    elif temp < sensor.highLimit - sensor.hysteresis and temp > sensor.lowLimit + sensor.hysteresis:
        sensor.triggerObject.value = False

# def purgeTemporaryFiles(context):
    # global alarmDaemon
    # age = context.getArgument('maxAge', '30')
    # age = int(age)
    # alarmDaemon.purgeTemporaryFiles(age)



#!/usr/bin/python3

# Copyright (C) 2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

# Check that pyknx is present as soon as possible.
from homewatcher import ensurepyknx

from pyknx import logger
from homewatcher import configuration, timer
import subprocess
import tempfile
import os
import threading
import time
import smtplib
import ftplib
# from homewatcher import alarm

class ActivationCriterion(object):
    """ Represents a criterion that tells if a sensor can be activated. Criterion's logic has to be implemented in method named isValid(self). """
    def __init__(self, sensor, config):
        self.sensor = sensor
        self.config = config
        self.daemon = sensor.daemon

    @staticmethod
    def makeNew(forSensor, config):
        if config is None: return None

        if config.type == configuration.ActivationCriterion.Type.SENSOR:
            return SensorActivationCriterion(forSensor, config)
        elif config.type in (configuration.ActivationCriterion.Type.AND, configuration.ActivationCriterion.Type.OR):
            return BooleanActivationCriterion(forSensor, config)
        else:
            raise Exception('Unsupported criterion type "{0}".'.format(config.type))

class SensorActivationCriterion(ActivationCriterion):
    """ Criterion that allows sensor activation if another sensor is triggered or not. """
    def __init__(self, sensor, config):
        ActivationCriterion.__init__(self, sensor, config)

    @property
    def watchedSensor(self):
        return self.daemon.getSensorByName(self.config.sensorName)

    @property
    def shouldBeTriggered(self):
        return self.config.whenTriggered

    def isValid(self):
        return self.watchedSensor.isTriggered == self.shouldBeTriggered

class BooleanActivationCriterion(ActivationCriterion):
    def __init__(self, sensor, config):
        ActivationCriterion.__init__(self, sensor, config)
        self.children = []
        for childConfig in config.children:
            self.children.append(ActivationCriterion.makeNew(sensor, childConfig))

    @property
    def type(self):
        return self.config.type

    def isValid(self):
        for child in self.children:
            if self.config.type == configuration.ActivationCriterion.Type.AND:
                if not child.isValid(): return False
            elif self.config.type == configuration.ActivationCriterion.Type.OR:
                if child.isValid(): return True
            else:
                raise Exception('Unsupported criterion type "{0}".'.format(self.config.type))

        if self.config.type == configuration.ActivationCriterion.Type.AND:
            return True
        elif self.config.type == configuration.ActivationCriterion.Type.OR:
            return False
        else:
            raise Exception('Unsupported criterion type "{0}".'.format(self.config.type))

class SubprocessLoggerThread(threading.Thread):
    def __init__(self, process, readsStdout, name):
        threading.Thread.__init__(self, name='Logger thread')
        self.process = process
        self.readsStdout = readsStdout
        self.name = name

    def run(self):
        statusFormat = 'Logger thread for std{0} of {1} PID=<{2}> is {3}.'.format('out' if self.readsStdout else 'err', self.name, self.process.pid, '{0}')
        logger.reportInfo(statusFormat.format('started'))
        while self.process.returncode is None:
            stream = self.process.stdout if self.readsStdout else self.process.stderr
            line = stream.readline()
            if line == '':
                time.sleep(1)
                continue

            line = line.rstrip('\n')
            log = '[{0}.{3} pid={1}] {2}'.format(self.name, self.process.pid, line, 'out' if self.readsStdout else 'err')
            logger.reportInfo(log)
        logger.reportInfo(statusFormat.format('terminated'))

class Sensor(object):
    def __init__(self, daemon, config):
        # Classes cannot be instanciated!
        if config.isClass:
            raise Exception('Sensor config {0} represents a sensor class. It cannot be used to instanciate a sensor.'.format(config.name))

        self._daemon = daemon
        self._config = config
        self.linknx = daemon.linknx
        self._isTriggered = False # Sensor behaviour is implemented in getUpdatedTriggerState.
        self._activationTimer = None
        self._enabledObject = self.linknx.getObject(config.enabledObjectId)
        self._watchedObject = self.linknx.getObject(config.watchedObjectId)
        self._persistenceObject = self.linknx.getObject(config.persistenceObjectId)
        self.alert = self._daemon.getAlertByName(config.alertName)
        self.activationCriterion = ActivationCriterion.makeNew(self, config.activationCriterion)

    def isRequiredByCurrentMode(self):
        if self._daemon._isTerminated: return False

        # Simple case: the sensor is specified by its name in config.
        return self.name in self._daemon.currentMode.sensorNames

        # # Advanced case: one of the sensor base classes is specified by its name
        # # in config.
        # baseClassesNames = set(self.getInheritedClassNames())
# 
        # return len(baseClassesNames.intersection(self._daemon.currentMode.sensorNames)) > 0

    @property
    def daemon(self):
        return self._daemon

    @property
    def config(self):
        return self._config

    @property
    def name(self):
        return self._config.name

    @property
    def description(self):
        return self._config.description

    @property
    def type(self):
        return self._config.type

    @property
    def watchedObject(self):
        return self._watchedObject

    @property
    def watchedObjectId(self):
        return self._config.watchedObjectId

    @property
    def isTriggered(self):
        return self._isTriggered

    def isNotTriggered(self):
        return not self.isTriggered

    @property
    def persistenceObject(self):
        return self._persistenceObject

    @property
    def isInPrealert(self):
        return self in self.alert.sensorsInPrealert

    @property
    def isAlertActive(self):
        return self in self.alert.sensorsInAlert

    # @property
    # def isInhibited(self):
        # return self.alert.isInhibited

    # @isAlertActive.setter
    # def isAlertActive(self, value):
        # # No change.
        # if self.isAlertActive == value: return
# 
        # # Check that alert should not be discarded.
        # if value and self.isInhibited:
            # logger.reportWarning('{0}\' alert will not become active because its alert type is currently inhibited.'.format(self))
            # return
# 
        # # Change alert state.
        # self.setAlertActiveRaw(value)
# 
        # # Also set as active all sensors currently in prealert.
        # if value:
            # for sensor in self.daemon.sensors:
                # if sensor == self: continue
                # if sensor.isInPrealert and sensor.alertName == self.alertName:
                    # logger.reportInfo('{0} is in prealert, let it join the current alert immediately!'.format(sensor))
                    # sensor.setAlertActiveRaw(True)
# 
    # def setAlertActiveRaw(self, value):
        # """
        # Updates the internal alert active status without reevaluating daemon's alert status.
# 
        # value Whether alert is active.
        # returns True if state changed, False if sensor's alert state was already correct.
        # """
# 
        # # No change.
        # if value == self.isAlertActive: return False
# 
        # if value:
            # logger.reportInfo('Sensor {0}\'s alert is active.'.format(self))
            # # Do not set persistent alert immediately. Wait for alert to be
            # # implemented for that (or sensors will always consider alert has
            # # already been previously triggered).
            # # self.persistenceObject.value = True
        # else:
            # logger.reportInfo('Sensor {0}\'s alert is no more active. If an alert is current, this sensor will not participate.'.format(self.name))
        # if value:
            # self.alert.addSensorToAlert(self)
        # else:
            # self.alert.removeSensorFromAlert(self)
# 
        # if not value and not self._alertTimer is None:
            # self._alertTimer.stop()
            # self._alertTimer = None
        # return True

    def implementEmail(self, emailDescription):
        pass

    # def implementAlert(self, alertDescription):
        # logger.reportDebug('{0} is now implementing alert'.format(self))
        # if not self.isEnabled:
            # logger.reportDebug('{0} is not enabled, alert implementation is empty.'.format(self))
            # return
        # if not self.isAlertActive:
            # logger.reportDebug('{0}\'s alert is not active, alert implementation is empty.'.format(self))
            # return
# 
        # alertDescription.setCurrentSensor(self)
        # try:
            # self.implementAlertRaw(alertDescription)
        # finally:
            # alertDescription.setCurrentSensor(None)

    @property
    def isEnabled(self):
        """ Tell whether the sensor is currently active (i.e under surveillance and able to fire alarm) """
        return self._enabledObject.value

    @isEnabled.setter
    def isEnabled(self, value):
        logger.reportDebug('isEnabled={0} for {1}, activationTimer is {2}'.format(value, self, self._activationTimer))
        if not value:
            self.stopActivationTimer()

        if self._enabledObject.value == value: return
        self._enabledObject.value = value

        if value:
            try:
                if self.persistenceObject != None:
                    self.persistenceObject.value = False
                self.onEnabled()
            except Exception as e:
                self._enabledObject.value = False
                logger.reportException()
        else:
            # Sensor may currently be in alert.
            self.alert.removeSensorFromAlert(self)
            self.onDisabled()

        logger.reportInfo('Sensor {0} is now {1}'.format(self.name, 'enabled' if value else 'disabled'))

    # def hasAlertAlreadyBeenTriggered(self):
        # """ Tell whether this sensor has already participated in any of the current alerts. """
        # return self in self.daemon._triggeredSensorsSinceModeChanged

    def getInheritedClassNames(self):
            return [sensorConfig.name for sensorConfig in self.daemon.configuration.getInheritedClassNames(self.config)]

    def makePrealertTimer(self):
        def onPrealertEnded(timer):
            self.alert.notifySensorPrealertExpired(self)
        return timer.Timer(self, self.getPrealertDuration(), 'Prealert timer', onTimeoutReached=onPrealertEnded, onTerminated=None)

    def makeAlertTimer(self):
        def onAlertEnded(timer):
            self.alert.removeSensorFromAlert(self)
        return timer.Timer(self, self.getAlertDuration(), 'Alert timer', onTimeoutReached=None, onTerminated=onAlertEnded)

    def getUpdatedTriggerState(self):
        """
        Return the new trigger state.

        Implement sensor's logic here. The default implementation returns the watched object's value, which should suit most sensor types.
        """
        return self.watchedObject.value

    def getPrealertDuration(self):
        return self._config.prealertDuration.getForMode(self.daemon.currentMode.name)

    def getAlertDuration(self):
        return self._config.alertDuration.getForMode(self.daemon.currentMode.name)

    def getActivationDelay(self):
        delay = self._config.activationDelay.getForMode(self.daemon.currentMode.name)
        logger.reportDebug('getActivationDelay of {2} for {0}, currentMode={1}'.format(self, self.daemon.currentMode, delay))
        return delay

    def onEnabled(self):
        pass

    def onDisabled(self):
        pass

    def isActivationPending(self):
        return self._activationTimer != None and self._activationTimer.isAlive() and not self._activationTimer.isTerminating

    def _onActivationTimerTimeout(self, timer):
        self.isEnabled = True

    def _onActivationTimerIterate(self, timer):
        if self.activationCriterion != None and not self.activationCriterion.isValid():
            if not timer.isPaused:
                logger.reportInfo('Pausing activation timer for {0} because activation criterion is not satisfied.'.format(self))
            timer.pause()
        else:
            if timer.isPaused:
                # Restart activation delay.
                logger.reportInfo('Restarting activation timer for {0} because activation criterion is now satisfied.'.format(self))
                timer.reset()

    def startActivationTimer(self):
        # Already enabled.
        if self.isEnabled: return

        if self.isActivationPending():
            logger.reportInfo('An activation timer for {0} is already running. Cancel it and start a new one.'.format(self))
            self._activationTimer.stop()
        self._activationTimer = timer.Timer(self, self.getActivationDelay(), 'Activation timer', onTimeoutReached=self._onActivationTimerTimeout, onIterate=self._onActivationTimerIterate)
        self._activationTimer.start()

    def stopActivationTimer(self):
        if self._activationTimer != None:
            self._activationTimer.stop
            self._activationTimer = None

    def notifyWatchedObjectChanged(self):
        """ Notifies the sensor that its watched object's value has just changed. """
        newTriggeredState = self.getUpdatedTriggerState() # Depends on the concrete sensor class. Most of them will do nothing as trigger state IS the watched object state. But for FloatSensor for instance, trigger may take an hysteresis into account.
        if self._isTriggered == newTriggeredState: return
        self._isTriggered = newTriggeredState
        if self.isTriggered:
            logger.reportInfo('{0} is triggered.'.format(self.name))
            if self.isEnabled:
                self.alert.addSensorToAlert(self)
        else:
            # Nothing to do regarding alert here. If sensor was previously
            # triggered, alert will not end by simply releasing trigger.
            logger.reportInfo('{0}\'s trigger is released.'.format(self.name))

    def __repr__(self):
        return '{name} ({description})'.format(name=self.name, description=self.description if self.description != None else 'No description')

class FloatSensor(Sensor):
    def __init__(self, daemon, config):
        Sensor.__init__(self, daemon, config)

    def getUpdatedTriggerState(self):
        value = self.watchedObject.value
        low = self.config.lowerBound
        up = self.config.upperBound
        if self.isTriggered:
            # Apply hysteresis.
            if not low is None: low += self.config.hysteresis
            if not up is None: up -= self.config.hysteresis
        if not low is None and value <= low: return True
        if not up is None and value >= up: return True
        return False

    def notifyAlertsInhibited(self, isIntrusionInhibited, isFireInhibited):
        pass

    # def implementAlertRaw(self, alertDescription):
        # alertDescription.setTemperature()
        # currentTemp = self.temperatureObject.value
        # if self.highLimit != None and currentTemp > self.highLimit - self.hysteresis:
            # heatingErrorType = 'élevée'
            # threshold = self.highLimit
        # elif self.lowLimit != None:
            # heatingErrorType = 'basse'
            # threshold = self.lowLimit
        # alertDescription.addEmailText('Alerte température trop {2}, temp={0}°C, seuil d\'alerte={1}°C'.format(self.temperatureObject.value, threshold, heatingErrorType))

class BooleanSensor(Sensor):
    def __init__(self, daemon, config):
        Sensor.__init__(self, daemon, config)

    def getUpdatedTriggerState(self):
        if self.config.triggerValue:
            return self.watchedObject.value
        else:
            return not self.watcheObject.value



#!/usr/bin/python3

# Copyright (C) 2012-2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

"""
Standalone script used as a launcher for the Homewatcher daemon.
"""

# Check that pyknx is present as soon as possible.
from homewatcher import ensurepyknx

from pyknx import communicator, linknx, logger
from homewatcher import configuration
import argparse
import sys
import logging
import os

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument('homewatcherConfig', help='use HWCONF as homewatcher configuration.', metavar='HWCONF')
    parser.add_argument('-d', '--daemonize', help='ask daemon to detach and run as a background daemon.', action='store_true', default=False)
    parser.add_argument('--pid-file', dest='pidFile', help='write the PID of the daemon process to PIDFILE.', metavar='PIDFILE')
    parser.add_argument('--log-file', dest='logFile', help='output daemon\'s activity to LOGFILE rather than to standard output.', metavar='LOGFILE', default=None)
    parser.add_argument('-v', '--verbosity', dest='verbosityLevel', help='set verbosity level.', metavar='LEVEL', choices=[l.lower() for l in logger.getLevelsToString()], default='info')

    args = parser.parse_args()

    # Configure logger.
    logger.initLogger(None, args.verbosityLevel.upper())

    # The homewatcher daemon is represented by an instance of
    # a pyknx.communicator.Communicator that runs with an "user script" dedicated to
    # interfacing linknx with homewatcher's capabilities.
    # First: read homewatcher config to read the linknx server url.
    # Second: start pyknxcommunicator with homewatcher's user script.
    logger.reportInfo('Reading config file {file}'.format(file=args.homewatcherConfig))
    config = configuration.Configuration.parseFile(args.homewatcherConfig)
    userScript = os.path.join(os.path.dirname(configuration.__file__), 'linknxuserfile.py')
    logger.reportDebug('Pyknx\'s user script for homewatcher is {script}'.format(script=userScript))
    userScriptArgs = {'hwconfig' : config}
    services = config.servicesRepository
    communicatorAddress=('localhost', services.daemon.port)
    logger.reportInfo('Starting Homewatcher at {communicatorAddr}, linked to linknx at {linknxAddr}'.format(communicatorAddr=communicatorAddress, linknxAddr=services.linknx.address))
    linknx = linknx.Linknx(services.linknx.host, services.linknx.port)
    communicator.Communicator.run(linknxAddress=linknx.address, userFile=userScript, communicatorAddress=communicatorAddress, userScriptArgs=userScriptArgs, verbosityLevel=args.verbosityLevel, logFile=args.logFile, daemonizes=args.daemonize, pidFile=args.pidFile)



#!/usr/bin/python3

# Copyright (C) 2012-2014 Cyrille Defranoux
#
# This file is part of Homewatcher.
#
# Homewatcher is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Homewatcher is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Homewatcher. If not, see <http://www.gnu.org/licenses/>.
#
# For any question, feature requests or bug reports, feel free to contact me at:
# knx at aminate dot net

from distutils.core import setup
import pip
import sys
import homewatcher

def installRequirement(requirementName):
    print('************************* IMPORTANT ******************************'.format(requirementName))
    print('The {0} package is required and is not installed on this machine.'.format(requirementName))
    print('You should install {0} first using for instance "pip3 install {0}"'.format(requirementName))
    print('******************************************************************'.format(requirementName))
    # print('Do you want to abort installation of homewatcher in order to complete this required step? (yes/no)')
    # answer = None
    # while not answer:
        # answer = sys.stdin.readline().lower().strip()
        # if not answer in ('yes', 'no'):
            # print('Please answer either yes or no.')
            # answer = None
    # if answer == 'yes':
        # exit(2)
    # args = ['install']
    # if canGetPreVersion: args.append('--pre')
    # args.append(requirementName)
    # pip.main(args)

if sys.version_info.major < 3:
    print('This package is compatible with Python 3 and above.')
    exit(4)

# Install pyknx if required.
try:
    import pyknx
except ImportError:
    installRequirement('pyknx')

# # Install lxml.
# try:
    # import lxml
# except ImportError:
    # installRequirement('lxml')


setup(name='homewatcher',
      version=homewatcher.__version__,
      description='Alarm system built on top of Linknx',
      long_description=''.join(open('README.md').readlines()),
      author='Cyrille Defranoux',
      author_email='knx@aminate.net',
      maintainer='Cyrille Defranoux',
      maintainer_email='knx@aminate.net',
      license='GNU Public General License',
      url='https://github.com/2franix/homewatcher/',
      requires=['pyknx (>=2.0)', 'lxml'],
      packages=['homewatcher'],
      data_files=[('.', ['README.md'])],
      scripts=['hwconf.py', 'hwdaemon.py', 'hwresolve.py', 'hwversion.py'])
